package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.entity.DrugConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.DrugConfigService;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.DrugConfigListVO;
import com.diagbot.vo.DrugConfigPageVO;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/7/29 15:04
 */
@Component
public class DrugConfigFacade {
    @Autowired
    private DrugConfigService drugConfigService;

    /**
     * 判断是否已存在
     *
     * @param drugConfig
     * @return
     */
    public Boolean isExistRecord(DrugConfig drugConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        QueryWrapper<DrugConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", drugConfig.getHisName())
                .eq("unique_name", drugConfig.getUniqueName());
        if (StringUtil.isBlank(drugConfig.getForm())) {
            queryWrapper.and(i -> i.isNull("form")
                    .or()
                    .eq("form", ""));
        } else {
            queryWrapper.eq("form", drugConfig.getForm());
        }
        DrugConfig oldRecord = drugConfigService.getOne(queryWrapper, false);
        if (oldRecord != null) {
            return true;
        }
        return false;
    }

    /**
     * 保存记录-单条
     *
     * @param drugConfig
     * @return
     */
    public Boolean saveOrUpdateRecord(DrugConfig drugConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        drugConfig.setHospitalId(Long.valueOf(hospitalId));
        drugConfig.setModifier(userId);
        drugConfig.setGmtModified(now);
        QueryWrapper<DrugConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", drugConfig.getHisName())
                .eq("unique_name", drugConfig.getUniqueName());
        if (StringUtil.isBlank(drugConfig.getForm())) {
            queryWrapper.and(i -> i.isNull("form")
                    .or()
                    .eq("form", ""));
        } else {
            queryWrapper.eq("form", drugConfig.getForm());
        }
        DrugConfig oldRecord = drugConfigService.getOne(queryWrapper, false);
        if (oldRecord != null) {
            drugConfig.setId(oldRecord.getId());
        }
        //新增数据
        if (drugConfig.getId() == null) {
            drugConfig.setCreator(userId);
            drugConfig.setGmtCreate(now);
        }
        if (drugConfig.getIsDeleted() == null) {
            drugConfig.setIsDeleted(IsDeleteEnum.N.getKey());
        }
        drugConfigService.saveOrUpdate(drugConfig);
        return true;
    }

    /**
     * 保存记录-批量
     *
     * @param drugConfigListVO
     * @return
     */
    public Boolean saveOrUpdateRecords(DrugConfigListVO drugConfigListVO) {
        if (ListUtil.isEmpty(drugConfigListVO.getDrugConfigList())) {
            return false;
        }
        return saveOrUpdateRecords(drugConfigListVO.getDrugConfigList());
    }

    /**
     * 批量保存
     *
     * @param drugConfigList
     * @return
     */
    public Boolean saveOrUpdateRecords(List<DrugConfig> drugConfigList) {
        if (ListUtil.isEmpty(drugConfigList)) {
            return false;
        }
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //数据不完整的不保存
        //过滤外部名称或公表名为空的数据
        drugConfigList = drugConfigList
                .stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName()))
                .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());
        List<String> hisNames = drugConfigList
                .stream()
                .map(i -> i.getHisName())
                .collect(Collectors.toList());
        List<String> uniqueNames = drugConfigList
                .stream()
                .map(i -> i.getUniqueName())
                .collect(Collectors.toList());

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, Map<String, Long>>> configMap
                = getConfigMap(Long.valueOf(hospitalId), hisNames, uniqueNames);
        drugConfigList.forEach(drugConfig -> {
            drugConfig.setHospitalId(Long.valueOf(hospitalId));
            drugConfig.setModifier(userId);
            drugConfig.setGmtModified(now);
            String form = StringUtil.isBlank(drugConfig.getForm()) ? "" : drugConfig.getForm();
            if (drugConfig.getId() == null) {
                if (configMap.get(drugConfig.getHisName()) != null
                        && configMap.get(drugConfig.getHisName()).get(form) != null
                        && configMap.get(drugConfig.getHisName()).get(form).get(drugConfig.getUniqueName()) != null) {
                    deleteIds.add(configMap.get(drugConfig.getHisName()).get(form).get(drugConfig.getUniqueName()));
                }
                drugConfig.setCreator(userId);
                drugConfig.setGmtCreate(now);
            }
            if (drugConfig.getIsDeleted() == null) {
                drugConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });

        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        drugConfigService.saveOrUpdateBatch(drugConfigList);
        return true;
    }

    /**
     * 删除记录-单条
     *
     * @param idVO
     * @return
     */
    public Boolean deleteRecord(IdVO idVO) {
        UpdateWrapper<DrugConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", idVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        drugConfigService.removeById(idVO.getId());
        return true;
    }

    /**
     * 删除记录-批量
     *
     * @param idListVO
     * @return
     */
    public Boolean deleteRecords(IdListVO idListVO) {
        if (ListUtil.isEmpty(idListVO.getIds())) {
            return false;
        }
        UpdateWrapper<DrugConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", idListVO.getIds())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        drugConfigService.removeByIds(idListVO.getIds());
        return true;
    }

    /**
     * 分页查询
     *
     * @param drugConfigPageVO
     * @return
     */
    public IPage<DrugConfig> getPage(DrugConfigPageVO drugConfigPageVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        drugConfigPageVO.setHospitalId(Long.valueOf(hospitalId));
        return drugConfigService.getPage(drugConfigPageVO);
    }

    /**
     * 数据导入
     *
     * @param file
     */
    public void importExcel(MultipartFile file) {
        List<DrugConfig> drugConfigList = ExcelUtils.importExcel(file, 0, 1, DrugConfig.class);
        if (ListUtil.isNotEmpty(drugConfigList)) {
            saveOrUpdateRecords(drugConfigList);
        }
    }

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return Map<hisName,Map<form,Map<uniqueName,id>>>
     */
    public Map<String, Map<String,Map<String, Long>>> getConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Map<String, Long>>> retMap = new HashMap<>();
        QueryWrapper<DrugConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<DrugConfig> records = drugConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<DrugConfig>> hisNameMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<DrugConfig>> entry : hisNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                Map<String, Map<String, Long>> formMap = new HashMap<>();
                entry.getValue().forEach(i -> {
                    if (StringUtil.isBlank(i.getForm())) {
                        i.setForm("");
                    }
                });
                Map<String, List<DrugConfig>> subMap
                        = EntityUtil.makeEntityListMap(entry.getValue(), "form");
                for (Map.Entry<String, List<DrugConfig>> subEntry : subMap.entrySet()) {
                    if (ListUtil.isNotEmpty(subEntry.getValue())) {
                        formMap.put(subEntry.getKey(),
                                EntityUtil.makeMapWithKeyValue(subEntry.getValue(), "uniqueName", "id"));
                    }
                }
                retMap.put(entry.getKey(), formMap);
            }
        }
        return retMap;
    }

    /**
     * 获取映射关系
     * Map<uniqueName,Map<form,Map<hisName,id>>>
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String,Map<String,Map<String,Long>>> getUniqueNameConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Map<String, Long>>> retMap = new HashMap<>();
        QueryWrapper<DrugConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<DrugConfig> records = drugConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        records.forEach(i -> {
            if (StringUtil.isBlank(i.getForm())) {
                i.setForm("");
            }
        });
        Map<String, List<DrugConfig>> uniqueNameMap = EntityUtil.makeEntityListMap(records, "uniqueName");
        for (Map.Entry<String, List<DrugConfig>> entry : uniqueNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                Map<String, Map<String, Long>> formMap = new HashMap<>();
                Map<String, List<DrugConfig>> subMap
                        = EntityUtil.makeEntityListMap(entry.getValue(), "form");
                for (Map.Entry<String, List<DrugConfig>> subEntry : subMap.entrySet()) {
                    if (ListUtil.isNotEmpty(subEntry.getValue())) {
                        formMap.put(subEntry.getKey(),
                                EntityUtil.makeMapWithKeyValue(subEntry.getValue(), "hisName", "id"));
                    }
                }
                retMap.put(entry.getKey(), formMap);
            }
        }
        return retMap;
    }

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return Map<hisName,Map<uniqueName,id>>
     */
    public Map<String, Map<String,Long>> getConfigMapWithoutForm(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<DrugConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<DrugConfig> records = drugConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<DrugConfig>> hisNameMap
                = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<DrugConfig>> entry : hisNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(),
                        EntityUtil.makeMapWithKeyValue(records, "uniqueName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 获取映射关系
     * Map<uniqueName,Map<hisName,id>>
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String,Map<String,Long>> getUniqueNameConfigMapWithoutForm(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<DrugConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<DrugConfig> records = drugConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        records.forEach(i -> {
            if (StringUtil.isBlank(i.getForm())) {
                i.setForm("");
            }
        });
        Map<String, List<DrugConfig>> uniqueNameMap
                = EntityUtil.makeEntityListMap(records, "uniqueName");
        for (Map.Entry<String, List<DrugConfig>> entry : uniqueNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(),
                        EntityUtil.makeMapWithKeyValue(records, "hisName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 数据导出
     *
     * @param response
     */
    public void exportExcel(HttpServletResponse response) {
        QueryWrapper<DrugConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", SysUserUtils.getCurrentHospitalID());
        List<DrugConfig> records = drugConfigService.list(queryWrapper);
        String fileName = "药品映射.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", DrugConfig.class, fileName, response, 12.8f);
    }
}