package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.entity.PacsConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.PacsConfigService;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.PacsConfigListVO;
import com.diagbot.vo.PacsConfigPageVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/7/29 15:03
 */
@Component
public class PacsConfigFacade {
    @Autowired
    private PacsConfigService pacsConfigService;

    /**
     * 判断是否已存在
     *
     * @param pacsConfig
     * @return
     */
    public Boolean isExistRecord(PacsConfig pacsConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        QueryWrapper<PacsConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", pacsConfig.getHisName())
                .eq("unique_name", pacsConfig.getUniqueName());
        PacsConfig oldRecord = pacsConfigService.getOne(queryWrapper, false);
        if (oldRecord != null) {
            return true;
        }
        return false;
    }

    /**
     * 保存记录-单条
     *
     * @param pacsConfig
     * @return
     */
    public Boolean saveOrUpdateRecord(PacsConfig pacsConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        pacsConfig.setHospitalId(Long.valueOf(hospitalId));
        pacsConfig.setModifier(userId);
        pacsConfig.setGmtModified(now);
        QueryWrapper<PacsConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", pacsConfig.getHisName())
                .eq("unique_name", pacsConfig.getUniqueName());
        PacsConfig oldRecord = pacsConfigService.getOne(queryWrapper, false);
        if (oldRecord != null) {
            pacsConfig.setId(oldRecord.getId());
        }
        //新增数据
        if (pacsConfig.getId() == null) {
            pacsConfig.setCreator(userId);
            pacsConfig.setGmtCreate(now);
        }
        if (pacsConfig.getIsDeleted() == null) {
            pacsConfig.setIsDeleted(IsDeleteEnum.N.getKey());
        }
        pacsConfigService.saveOrUpdate(pacsConfig);
        return true;
    }

    /**
     * 保存记录-批量
     *
     * @param pacsConfigListVO
     * @return
     */
    public Boolean saveOrUpdateRecords(PacsConfigListVO pacsConfigListVO) {
        if (ListUtil.isEmpty(pacsConfigListVO.getPacsConfigList())) {
            return false;
        }
        return saveOrUpdateRecords(pacsConfigListVO.getPacsConfigList());
    }

    /**
     * 批量保存
     *
     * @param pacsConfigList
     * @return
     */
    public Boolean saveOrUpdateRecords(List<PacsConfig> pacsConfigList) {
        if (ListUtil.isEmpty(pacsConfigList)) {
            return false;
        }
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //数据不完整的不保存
        //过滤外部名称或公表名为空的数据
        pacsConfigList = pacsConfigList
                .stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName()))
                .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, Long>> configMap
                = getConfigMap(Long.valueOf(hospitalId), null, null);
        pacsConfigList.forEach(pacsConfig -> {
            pacsConfig.setHospitalId(Long.valueOf(hospitalId));
            pacsConfig.setModifier(userId);
            pacsConfig.setGmtModified(now);
            if (pacsConfig.getId() == null) {
                if (configMap.containsKey(pacsConfig.getHisName())) {
                    deleteIds.add(configMap.get(pacsConfig.getHisName()).get(pacsConfig.getUniqueName()));
                }
                pacsConfig.setCreator(userId);
                pacsConfig.setGmtCreate(now);
            }
            if (pacsConfig.getIsDeleted() == null) {
                pacsConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });
        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        pacsConfigService.saveOrUpdateBatch(pacsConfigList);
        return true;
    }

    /**
     * 删除记录-单条
     *
     * @param idVO
     * @return
     */
    public Boolean deleteRecord(IdVO idVO) {
        UpdateWrapper<PacsConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", idVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        pacsConfigService.removeById(idVO.getId());
        return true;
    }

    /**
     * 删除记录-批量
     *
     * @param idListVO
     * @return
     */
    public Boolean deleteRecords(IdListVO idListVO) {
        if (ListUtil.isEmpty(idListVO.getIds())) {
            return false;
        }
        UpdateWrapper<PacsConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", idListVO.getIds())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        pacsConfigService.removeByIds(idListVO.getIds());
        return true;
    }

    /**
     * 分页查询
     *
     * @param pacsConfigPageVO
     * @return
     */
    public IPage<PacsConfig> getPage(PacsConfigPageVO pacsConfigPageVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        pacsConfigPageVO.setHospitalId(Long.valueOf(hospitalId));
        return pacsConfigService.getPage(pacsConfigPageVO);
    }

    /**
     * 数据导入
     *
     * @param file
     */
    public void importExcel(MultipartFile file) {
        List<PacsConfig> pacsConfigList = ExcelUtils.importExcel(file, 0, 1, PacsConfig.class);
        if (ListUtil.isNotEmpty(pacsConfigList)) {
            saveOrUpdateRecords(pacsConfigList);
        }
    }

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<PacsConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<PacsConfig> records = pacsConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<PacsConfig>> configMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<PacsConfig>> entry : configMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "uniqueName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 获取映射关系
     * Map<uniqueName,Map<hisName,id>>
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String,Map<String,Long>> getUniqueNameConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<PacsConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<PacsConfig> records = pacsConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }

        Map<String, List<PacsConfig>> uniqueNameMap = EntityUtil.makeEntityListMap(records, "uniqueName");
        for (Map.Entry<String, List<PacsConfig>> entry : uniqueNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "hisName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 数据导出
     *
     * @param response
     */
    public void exportExcel(HttpServletResponse response) {
        QueryWrapper<PacsConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", SysUserUtils.getCurrentHospitalID());
        List<PacsConfig> records = pacsConfigService.list(queryWrapper);
        String fileName = "检查映射.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", PacsConfig.class, fileName, response, 12.8f);
    }
}
